/*
  THKD
  Written by Jan Engelhardt <jengelh [at] gmx de>, 2005 - 2006

  This program is in the public domain.
  Do what you want, but give credit where due.
  And do not blame the author if anything breaks.
*/
#include <linux/buffer_head.h>
#include <linux/err.h>
#include <linux/fs.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/kthread.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/sched.h>
#include <linux/smp_lock.h>
#include <linux/vmalloc.h>
#define PREFIX          __stringify(KBUILD_BASENAME) ": "
#ifndef SEEK_SET
#    define SEEK_SET    1
#endif

// Functions
static int touch_thread(void *);
static struct file *dev_open(const char *);
static void dev_touch(struct file *, struct block_device *);

// Variables
static struct task_struct *touch_task = NULL;
static char *Device     = "/dev/hda";
static long BIGBUFSZ    = 128 * 1024;
static long BLKSZ       = 4096;
static long Interval    = 3 * HZ;

// Module stuff
MODULE_AUTHOR("Jan Engelhardt <jengelh [at] gmx de>");
MODULE_DESCRIPTION("TOSHIBA MK2003GAH harddisk auto-parking workaround");
MODULE_LICENSE("GPL and additional rights"); // PD, but GPL to make kmod shut up
module_param(BIGBUFSZ, long, S_IRUGO);
module_param(BLKSZ,    long, S_IRUGO | S_IWUSR);
module_param(Interval, long, S_IRUGO | S_IWUSR);
module_param(Device,  charp, S_IRUGO);

//-----------------------------------------------------------------------------
static __init int thkd_init(void)
{
	touch_task = kthread_run(touch_thread, Device, "thkd");
	if(IS_ERR(touch_task) || unlikely(touch_task == NULL)) {
		printk(KERN_WARNING PREFIX "Could not start thread: %ld\n",
			   PTR_ERR(touch_task));
		return PTR_ERR(touch_task);
	}

	return 0;
}

static __exit void thkd_exit(void)
{
	kthread_stop(touch_task);
	return;
}

module_init(thkd_init);
module_exit(thkd_exit);

static int touch_thread(void *dev_name)
{
	struct file *filp;

	if((filp = dev_open(dev_name)) == NULL) {
		printk(KERN_INFO PREFIX "Module unloaded while looking "
		       "for device\n");
		return 0;
	}

	dev_touch(filp, filp->f_dentry->d_inode->i_bdev);
	filp_close(filp, NULL);
	return 0;
}

static struct file *dev_open(const char *dev_name)
{
	struct file *filp = NULL;

	while(!kthread_should_stop()) {
		filp = filp_open(dev_name, O_RDONLY | O_NOATIME, 0400);
		if(!IS_ERR(filp) && filp != NULL)
			return filp;
		printk(KERN_INFO PREFIX "Could not open %s: %ld, retry in "
		       "5 seconds\n", dev_name, PTR_ERR(filp));
		set_current_state(TASK_UNINTERRUPTIBLE);
		schedule_timeout(Interval);
	}

	return NULL;
}

static void dev_touch(struct file *filp, struct block_device *bdev)
{
	mm_segment_t oldfs;
	loff_t ppos = 0;
	ssize_t ret;
	char *buf;

	if((buf = vmalloc(BIGBUFSZ)) == NULL) {
		printk(KERN_WARNING PREFIX "Memory allocation failure\n");
		return;
	}

	printk(KERN_INFO PREFIX "Touch daemon parameters: interval %ld "
	       "jiffies, block size %ld bytes (max %ld)\n",
	       Interval, BLKSZ, BIGBUFSZ);

	while(!kthread_should_stop()) {
		oldfs = get_fs();
		set_fs(KERNEL_DS);
		ret = vfs_read(filp, buf, (BLKSZ > BIGBUFSZ) ?
		      BIGBUFSZ : BLKSZ, &ppos);
		set_fs(oldfs);

		if(ret < 0) {
			printk(KERN_WARNING PREFIX "vfs_read() returned %zd\n", ret);
			return;
		} else if(ret == 0) {
			vfs_llseek(filp, 0, SEEK_SET);
			continue;
		}

		lock_kernel();
		//fsync_bdev(bdev);
		invalidate_bdev(bdev, 0);
		unlock_kernel();

		set_current_state(TASK_INTERRUPTIBLE);
		schedule_timeout(Interval);
	}

	vfree(buf);
	return;
}

//=============================================================================
