
#ifndef _RSYSRQ_H
#define _RSYSRQ_H

#include <asm/byteorder.h>

#ifndef __KERNEL__
#include <sys/types.h>

typedef __uint8_t	u8;
typedef __uint32_t	u32;
typedef __uint64_t	u64;

#if __BYTE_ORDER == __LITTLE_ENDIAN
#define	be32_to_cpu(x)	__swab32((x))
#define	cpu_to_be32(x)	__swab32((x))
#elif __BYTE_ORDER == __BIG_ENDIAN
#define	be32_to_cpu(x)	((u32)(x))
#define	cpu_to_be32(x)	((u32)(x))
#else
#error "Unknown endian"
#endif

#endif

#define SHA1_DIGEST_SIZE	20
void sha1_hash( const unsigned char *data, unsigned int len, unsigned char *hash );

typedef u32	my_time_t;

struct rsysrq_s {
	unsigned char request_string[SHA1_DIGEST_SIZE];
	my_time_t timestamp;
};

/* create encrypted request */
void rsysrq_create( struct rsysrq_s *request, const char cmd, const unsigned char *password, unsigned char (*rnd_gen)(void), my_time_t timestamp );

/* decrypt and extract command from request if valid, otherwise return 0 */
unsigned char rsysrq_extract( struct rsysrq_s *request, const unsigned char *password, my_time_t now, my_time_t tolerance );

#endif /* _RSYSRQ_H */

