/*
 * Transparent proxy support for Linux/iptables
 *
 * Copyright (c) 2002-2004 BalaBit IT Ltd.
 * Author: Balzs Scheidler
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#include <linux/module.h>
#include <linux/types.h>
#include <linux/netfilter.h>
#include <linux/netfilter_ipv4/ip_conntrack.h>
#include <linux/netfilter_ipv4/ip_tables.h>

static int
match(const struct sk_buff *skb,
      const struct net_device *in,
      const struct net_device *out,
      const struct xt_match *match,
      const void *matchinfo,
      int offset,
      unsigned int protoff,
      int *hotdrop)
{
	enum ip_conntrack_info ctinfo;
	struct ip_conntrack *ct;

	ct = ip_conntrack_get((struct sk_buff *)skb, &ctinfo);
	if (ct)
		return ct->status & IPS_TPROXY;
	else
		return 0;
}

static int check(const char *tablename,
		 const void *ip_void,
		 const struct xt_match *match,
		 void *matchinfo,
		 unsigned int hook_mask)
{
	return 1;
}

static struct ipt_match tproxy_match = {
	.name		= "tproxy",
	.match		= &match,
	.matchsize	= 0,
	.checkentry	= &check,
	.me		= THIS_MODULE,
};

static int __init init(void)
{
	return ipt_register_match(&tproxy_match);
}

static void __exit fini(void)
{
	ipt_unregister_match(&tproxy_match);
}

module_init(init);
module_exit(fini);
MODULE_LICENSE("GPL");
MODULE_AUTHOR("Balzs Scheidler <bazsi@balabit.hu>");
MODULE_DESCRIPTION("Netfilter transparent proxy iptables match module.");

